﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using System.Linq;

  using HeuristicLab.BenchmarkSuite.ERC;
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class Data : NamedItem {

    public Data() { }

    [StorableConstructor]
    protected Data(bool deserializing) : base(deserializing) { }

    public Data(Data origin, Cloner cloner) : base(origin, cloner) {
      Name = origin.Name;
      Description = origin.Description;
      OriginalTrainingCount = origin.OriginalTrainingCount;
      OriginalTestCount = origin.OriginalTestCount;
      BestResult = origin.BestResult;
      WorstResult = origin.WorstResult;
      InputArgumentTypes = origin.InputArgumentTypes;
      OutputArgumentTypes = origin.OutputArgumentTypes;
      Examples = origin.Examples.ToArray();
      EnabledDataTypes = origin.EnabledDataTypes;
      this.ProblemErcOptions = cloner.Clone(origin.ProblemErcOptions);
    }

    [Storable]
    public string Name { get; set; }
    [Storable]
    public string Description { get; set; }
    [Storable]
    public int OriginalTrainingCount { get; set; }
    [Storable]
    public int OriginalTestCount { get; set; }
    [Storable]
    public int BestResult { get; set; }
    [Storable]
    public int WorstResult { get; set; }
    [Storable]
    public ExampleArgumentType[] InputArgumentTypes { get; set; }
    [Storable]
    public ExampleArgumentType[] OutputArgumentTypes { get; set; }
    public int TotalArgumentCount { get { return InputArgumentTypes.Length + OutputArgumentTypes.Length; } }
    [Storable]
    public Example[] Examples { get; set; }
    [Storable]
    public ProblemErcOptions ProblemErcOptions { get; set; }
    [Storable]
    public DataTypes EnabledDataTypes { get; set; }

    /// <summary>
    /// Max Size” gives the maximum number of instructions that can appear in an individual’s genome
    /// </summary>
    [Storable]
    public int MaxSize { get; set; }

    /// <summary>
    /// “Eval Limit” is the number of steps of the Push interpreter that are executed before stopping a
    /// program’s execution; programs halted in this way may still achieve good results if they print or
    /// return results before they are stopped
    /// </summary>
    [Storable]
    public int EvalLimit { get; set; }

    /// <summary>
    /// “Max Gens” gives the maximum number of generations in a single PushGP run.
    /// </summary>
    [Storable]
    public int MaxGenerations { get; set; }

    /// <summary>
    /// “Prog Eval Budget” is the maximum number of programs that will be evaluated before a run is terminated.
    /// </summary>
    [Storable]
    public int ProgEvalBudget { get; set; }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Data(this, cloner);
    }
  }
}
