﻿namespace HeuristicLab.BenchmarkSuite.Views {
  public static class ViewHelper {
    public static string StringifyInput(ExampleArgumentType type, Example example, string valueSeparator) {
      switch (type) {
        case ExampleArgumentType.Integer:
        case ExampleArgumentType.IntegerCollection: return string.Join(valueSeparator, example.InputInt);

        case ExampleArgumentType.Float:
        case ExampleArgumentType.FloatCollection: return string.Join(valueSeparator, example.InputFloat);

        case ExampleArgumentType.Bool: return string.Join(valueSeparator, example.InputBoolean);
        case ExampleArgumentType.Char: return string.Join(valueSeparator, example.InputChar);

        case ExampleArgumentType.String:
        case ExampleArgumentType.StringCollection: return string.Join(valueSeparator, example.InputString);
        default: return string.Empty;
      }
    }

    public static string StringifyOutput(ExampleArgumentType type, Example example, string valueSeparator) {
      switch (type) {
        case ExampleArgumentType.Integer:
        case ExampleArgumentType.IntegerCollection: return string.Join(valueSeparator, example.OutputInt);

        case ExampleArgumentType.Float:
        case ExampleArgumentType.FloatCollection: return string.Join(valueSeparator, example.OutputFloat);

        case ExampleArgumentType.Bool: return string.Join(valueSeparator, example.OutputBoolean);
        case ExampleArgumentType.Char: return string.Join(valueSeparator, example.OutputChar);

        case ExampleArgumentType.String:
        case ExampleArgumentType.StringCollection: return string.Join(valueSeparator, example.OutputString);
        default: return string.Empty;
      }
    }

    private const string IntegerColumnHeader = "int";
    private const string IntegersColumnHeader = "int[]";
    private const string DoubleColumnHeader = "double";
    private const string DoublesColumnHeader = "double[]";
    private const string BooleanColumnHeader = "bool";
    private const string CharColumnHeader = "char";
    private const string StringColumnHeader = "string";
    private const string StringsColumnHeader = "string[]";
    public static string GetHeaderTypeName(ExampleArgumentType type) {
      switch (type) {
        case ExampleArgumentType.Integer: return IntegerColumnHeader;
        case ExampleArgumentType.Float: return DoubleColumnHeader;
        case ExampleArgumentType.Bool: return BooleanColumnHeader;
        case ExampleArgumentType.Char: return CharColumnHeader;
        case ExampleArgumentType.String: return StringColumnHeader;
        case ExampleArgumentType.IntegerCollection: return IntegersColumnHeader;
        case ExampleArgumentType.FloatCollection: return DoublesColumnHeader;
        case ExampleArgumentType.StringCollection: return StringsColumnHeader;
        default: return string.Empty;
      }
    }
  }
}
