﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Encodings.IntegerVectorEncoding;
  using HeuristicLab.Optimization;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Creator;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;

  [Item("PushEncoding", "Describes a push program encoding.")]
  [StorableClass]
  public sealed class PushEncoding : Encoding<IPushProgramCreator> {

    [StorableConstructor]
    public PushEncoding(bool deserializing)
      : base(deserializing) {
    }

    public PushEncoding(PushEncoding origin, Cloner cloner)
      : base(origin, cloner) {

    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PushEncoding(this, cloner);
    }

    public override void ConfigureOperators(IEnumerable<IOperator> operators) {
      throw new NotImplementedException();
    }
  }

  public static class IndividualExtensionMethods {
    public static PushProgram PushProgram(this Individual individual, IReadOnlyList<string> enabledExpressions) {
      return individual.IntegerVector().MapToPushProgram(enabledExpressions);
    }

    public static PushProgram MapToPushProgram(this IntegerVector vector, IReadOnlyList<string> enabledExpressions) {
      var expressions = new Expression[vector.Length];
      for (var i = 0; i < vector.Length; i++) {
        expressions[i] = ExpressionTable.GetExpression(enabledExpressions[vector[i]]);
      }

      return new PushProgram(expressions);
    }
  }
}
